// Minecraft Standalone Client
// Copyright (C) 2009-2010 Colin Bartolome
// Licensed under the GPL. See LICENSE.txt for details.

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.prefs.*;

import javax.swing.*;

/** This is a wrapper for the Minecraft applet. It's unauthorized, so don't go crying to them if it breaks. */
public class Main extends JFrame
{
   private static final String TITLE = "Minecraft";
   
   private static final Preferences PREFERENCES = Preferences.userRoot().node("minecraft-indev");
   
   private static final Preferences BOOKMARKS = PREFERENCES.node("bookmarks");
   
   private static final String KEY_TEXTURE_PACK = "texturePack";
   
   private static final String KEY_LOGIN = "login";
   
   private static final String KEY_MAXIMIZED = "maximized";
   
   private static final String KEY_WIDTH = "width";
   
   private static final String KEY_HEIGHT = "height";
   
   private static final int DEFAULT_WIDTH = 854;
   
   private static final int DEFAULT_HEIGHT = 480;
   
   private String loginCookie = "";
   
   public static void main(String[] args)
   {
      new Main();
   }
   
   private Main()
   {
      System.out.println("Minecraft Standalone Client");
      System.out.println("Copyright (C) 2009-2010 Colin Bartolome");
      System.out.println("Licensed under the GPL. See LICENSE.txt for details.");
      
      new UpdateDialog(this);

      final MainApplet applet = new MainApplet(this);
      
      setTitle(TITLE);
      getContentPane().add(applet);
      getContentPane().setPreferredSize(getInitialSize());
      pack();
      setLocationRelativeTo(null);
      if (isInitiallyMaximized())
         setExtendedState(JFrame.MAXIMIZED_BOTH);
      setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
      setVisible(true);
      
      addWindowListener(new WindowAdapter()
      {
         @Override
         public void windowClosing(WindowEvent evt)
         {
            super.windowClosing(evt);
            System.out.println("stopping...");
            applet.stop();
            applet.destroy();
            saveSize();
         }
      });
      
      getContentPane().addComponentListener(new ComponentAdapter()
      {
         @Override
         public void componentResized(ComponentEvent evt)
         {
            applet.resize(evt.getComponent().getSize());
         }
      });
      
      getContentPane().addKeyListener(new KeyAdapter()
      {
         @Override
         public void keyTyped(KeyEvent evt)
         {
            System.out.println("key: " + evt);
         }
      });      
      
      new LoginDialog(this);
      
      new ServerDialog(this, applet);

      applet.init();
   }
   
   private static boolean isInitiallyMaximized()
   {
      return PREFERENCES.get(KEY_MAXIMIZED, null) != null;
   }
   
   private static Dimension getInitialSize()
   {
      String width = PREFERENCES.get(KEY_WIDTH, "");
      String height = PREFERENCES.get(KEY_HEIGHT, "");
      
      try
      {
         return new Dimension(Integer.parseInt(width), Integer.parseInt(height));
      }
      catch (NumberFormatException nfe)
      {
         return new Dimension(DEFAULT_WIDTH, DEFAULT_HEIGHT);
      }
   }
   
   private void saveSize()
   {
      if (getExtendedState() == JFrame.MAXIMIZED_BOTH)
      {
         PREFERENCES.put(KEY_MAXIMIZED, "yes");
      }
      else
      {
         PREFERENCES.remove(KEY_MAXIMIZED);
         PREFERENCES.put(KEY_WIDTH, Integer.toString(getContentPane().getWidth()));
         PREFERENCES.put(KEY_HEIGHT, Integer.toString(getContentPane().getHeight()));
      }
   }
   
   public String getTexturePack()
   {
      return PREFERENCES.get(KEY_TEXTURE_PACK, null);
   }
   
   public void setTexturePack(String texturePack)
   {
      if (texturePack == null)
         PREFERENCES.remove(KEY_TEXTURE_PACK);
      else
         PREFERENCES.put(KEY_TEXTURE_PACK, texturePack);
   }
   
   public String getLoginCookie()
   {
      return PREFERENCES.get(KEY_LOGIN, loginCookie);
   }
   
   public void setLoginCookie(String username, String password, boolean save)
   {
      loginCookie = username + "|" + password;
      
      if (save)
         PREFERENCES.put(KEY_LOGIN, loginCookie);
      else
         PREFERENCES.remove(KEY_LOGIN);
   }
   
   public static Map<String, String> getBookmarks()
   {
      Map<String, String> bookmarks = new TreeMap<String, String>();
      
      try
      {
         for (String name : BOOKMARKS.keys())
            bookmarks.put(name, BOOKMARKS.get(name, ""));
      }
      catch (BackingStoreException bse)
      {
         System.err.println("Error reading bookmarks: " + bse.toString());
      }
      
      return bookmarks;
   }
   
   public static void saveBookmark(String name, String url)
   {
      if (url == null)
         BOOKMARKS.remove(name);
      else
         BOOKMARKS.put(name, url);
   }
   
   public void setStatus(final String text)
   {
      SwingUtilities.invokeLater(new Runnable()
      {
         @Override
         public void run()
         {
            setTitle(TITLE + text);
         }
      });
   }
}
